/*
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

#include "precompiled.hpp"
#include "logging/log.hpp"
#include "runtime/globals.hpp"
#include "runtime/os.hpp"
#include "runtime/safepointMechanism.hpp"
#include "utilities/globalDefinitions.hpp"

void SafepointMechanism::default_initialize() {
  const size_t page_size = os::vm_page_size();
  char* polling_page = os::reserve_memory(page_size, NULL, page_size);
  os::commit_memory_or_exit(polling_page, page_size, false, "Unable to commit Safepoint polling page");
  bool res = os::protect_memory(polling_page, page_size, os::MEM_PROT_READ);

  log_info(os)("SafePoint Polling address: " INTPTR_FORMAT, p2i(polling_page));
  os::set_polling_page((address)(polling_page));
}

void SafepointMechanism::initialize_serialize_page() {
  if (!UseMembar) {
    const size_t page_size = os::vm_page_size();
    char* serialize_page = os::reserve_memory(page_size, NULL, page_size);
    os::commit_memory_or_exit(serialize_page, page_size, false, "Unable to commit memory serialization page");
    log_info(os)("Memory Serialize Page address: " INTPTR_FORMAT, p2i(serialize_page));
    os::set_memory_serialize_page((address)(serialize_page));
  }
}

void SafepointMechanism::initialize() {
  pd_initialize();
  initialize_serialize_page();
}
